<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads
 *
 * @package   jchoptimize/wordpress-platform
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

defined( '_WP_EXEC' ) or die( 'Restricted access' );

use JchOptimize\Core\Admin\Ajax\Ajax;
use JchOptimize\Platform\Cache;

include JCH_PLUGIN_DIR . 'admin/form.php';
include JCH_PLUGIN_DIR . 'admin/assets.php';
include JCH_PLUGIN_DIR . 'admin/settings.php';

add_action( 'admin_menu', 'add_jch_optimize_menu' );

function add_jch_optimize_menu()
{
	$hook_suffix = add_options_page( __( 'JCH Optimize Settings', 'jch-optimize' ), 'JCH Optimize', 'manage_options', 'jchoptimize-settings',
		'jch_options_form' );

	add_action( 'admin_enqueue_scripts', 'jch_load_resource_files' );
	add_action( 'admin_head-' . $hook_suffix, 'jch_load_scripts' );
	add_action( 'load-' . $hook_suffix, 'jch_initialize_settings' );
}

add_action( 'admin_init', 'jch_register_options' );

function jch_register_options()
{
	register_setting( 'jch_options', 'jch_options', 'jch_options_validate' );
}

add_action( 'admin_bar_menu', 'jch_admin_bar_menu', 100 );

function jch_admin_bar_menu( $admin_bar )
{
	$args = array(
		'id'    => 'jch-optimize-parent',
		'title' => 'JCH Optimize'
	);

	$admin_bar->add_node( $args );

	$args = array(
		'parent' => 'jch-optimize-parent',
		'id'     => 'jch-optimize-cache',
		'title'  => 'Clean Cache',
		'href'   => add_query_arg( array( 'jch-admin-bar-task' => 'cleancache' ), admin_url( jch_get_current_admin_uri() ) )
	);

	$admin_bar->add_node( $args );
}

function jch_get_current_admin_uri()
{
	$uri = isset( $_SERVER['REQUEST_URI'] ) ? esc_url_raw( wp_unslash( $_SERVER['REQUEST_URI'] ) ) : '';
	$uri = preg_replace( '|^.*/wp-admin/|i', '', $uri );

	if ( ! $uri )
	{
		return '';
	}

	return $uri;
}

add_action( 'admin_init', 'jch_admin_bar_clean_cache' );

function jch_admin_bar_clean_cache()
{
	if ( isset( $_GET['jch-admin-bar-task'] ) && $_GET['jch-admin-bar-task'] = 'cleancache' )
	{
		try
		{
			$result = Cache::deleteCache();
		}
		catch ( Exception $e )
		{
		}

		if ( $result !== false )
		{
			$notice['success'] = __( 'The plugin\'s cache files were deleted successfully!', 'jch-optimize' );
		}
		else
		{
			$notice['error'] = __( 'An error occurred while trying to delete the plugin\'s cache files, probably already deleted!', 'jch-optimize' );
			//add_action('admin')
		}

		set_transient( 'jch-admin-bar-notice', $notice, 60 * 5 );

		wp_safe_redirect( remove_query_arg( array( 'jch-admin-bar-task' ), admin_url( jch_get_current_admin_uri() ) ) );
		exit;
	}

	if ( get_transient( 'jch-admin-bar-notice' ) )
	{
		add_action( 'admin_notices', 'jch_admin_bar_notice' );
	}
}

function jch_admin_bar_notice()
{
	$notices = get_transient( 'jch-admin-bar-notice' );

	foreach ( $notices as $type => $notice )
	{
		?>

            <div class="notice notice-<?php echo $type ?> is-dismissible">
                <p><?php echo $notice ?></p>
            </div>

		<?php
	}

	delete_transient( 'jch-admin-bar-notice' );
}

